import sqlite3
import parser
from shopdatabase import DatabaseController

ps = parser.Parser()
dbpath = ps.get_db_path()

class UserController(DatabaseController):
    """
    add_user(username, password, address, phoneNumber, email)
    add_staff(username, password, address, phoneNumber, email[, priv])
    get_user_info(username)
    get_staff_info(username)
    set_user(username, **kwargs) --> Please see the specific doc below.
    set_staff(username, **kwargs) --> Please see the specific doc below.
    delete_user(username)
    delete_staff(username)
    authenticate(username, password, flag) --> 0 as customer, 1 as staff.
    """
    def __init__(self):
        DatabaseController.__init__(self)

    def __sha512_hexdigest(self, password):
        """
        Private help function to return the hashed password used by sha512
        encryption method.
        
        Take 1 argument, password in plaintext.

        Returns the HEX hashed digest.
        """
        import hashlib
        h = hashlib.sha512(password)
        return h.hexdigest()

    def add_user(self, username, password, address, phoneNumber, email):
        """
        Adding a username to the database, but it won't allow duplicates.
        This method ignores the capital letters.

        Arguments taken: 2 arguments. username, password for customers.

        Returns True for success add, otherwise a False.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        test = c.execute("SELECT * FROM userdb WHERE username=(?)",
                (username.lower(),))
        try:
            test.next()
        except StopIteration:
            sha512 = self.__sha512_hexdigest(password)
            c.execute("INSERT INTO userdb VALUES (?, ?, ?, ?, ?, 0)",
                    (username.lower(), sha512, address, phoneNumber, email))
            conn.commit()
            conn.close()
            return True
        else:
            conn.close()
            return False

    def add_staff(self, username, password, address, phoneNumber, email, priv=1):
        """
        Adding a staff member, should only be accessable to system admins
        Require: username, password and privilage number.

        0 as system admin (full privilage)
        1 as normal staff (cashier)
        2 as stocking staff (add/remove DVD stock)
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        test = c.execute("SELECT * FROM staffdb WHERE username=(?)",
                (username.lower(),))
        try:
            test.next()
        except StopIteration:
            sha512 = self.__sha512_hexdigest(password)
            c.execute("INSERT INTO staffdb VALUES (?, ?, ?, ?, ?, ?)",
                    (username.lower(), sha512, address, phoneNumber, email, 
                        priv))
            conn.commit()
            conn.close()
            return True
        else:
            conn.close()
            return False

    def get_user_info(self, username):
        """
        Returns a list of tuple on a user if found in the order of:

        username, password, address, phoneNumber, email.
        0         1         2        3            4

        Otherwise return None
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        rawinfo = c.execute("SELECT * FROM userdb WHERE username=(?)",
                (username.lower(),))
        info = rawinfo.fetchone()
        conn.close()
        return info

    def get_staff_info(self, username):
        """
        Returns a tuple containing information on a staff in this given order:

        username, password, address, phoneNumber, email, priv
        0         1         2        3            4      5

        Otherwise it will return None
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        rawinfo = c.execute("SELECT * FROM staffdb WHERE username=(?)",
                (username.lower(),))
        info = rawinfo.fetchone()
        conn.close()
        return info

    def set_user(self, username, **kwargs):
        """
        Change everything about a user given a username, accessible by all
        staff members. Customer himself can change his own password, address
        and other info.

        It returns True for successful change, False as failure.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        test = c.execute("SELECT * FROM userdb WHERE username=(?)",
                (username.lower(),))

        # Check username exists
        try:
            test.next()
        except StopIteration:
            conn.close()
            return False

        if 'password' in kwargs.keys() and kwargs['password'] is not None:
            sha512 = self.__sha512_hexdigest(password)
            c.execute("UPDATE userdb SET password=(?) WHERE username=(?)",
                    (sha512, username.lower()))
        if 'address' in kwargs.keys() and kwargs['address'] is not None:
            c.execute("UPDATE userdb SET address=(?) WHERE username=(?)",
                    (address, username.lower()))
        if 'phoneNumber' in kwargs.keys() and kwargs['phoneNumber'] \
                is not None:
            c.execute("UPDATE userdb SET phone=(?) WHERE username=(?)",
                    (phone, username.lower()))
        if 'email' in kwargs.keys() and kwargs['email'] is not None:
            c.execute("UPDATE userdb SET email=(?) WHERE username=(?)",
                    (email, username.lower()))
        conn.commit()
        conn.close()
        return True

    def set_staff(self, username, **kwargs):
        """
        Change everything about a staff member by giving a username, accessible
        by only system admin.

        It returns
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        test = c.execute("SELECT * FROM userdb WHERE username=(?)",
                (username.lower(),))

        try:
            test.next()
        except StopIteration:
            return False

        if 'password' in kwargs.keys() and kwargs['password'] is not None:
            sha512 = self.__sha512_hexdigest(password)
            c.execute("UPDATE staffdb SET password=(?) WHERE username=(?)",
                    (sha512, username.lower()))
        if 'address' in kwargs.keys() and kwargs['address'] is not None:
            c.execute("UPDATE staffdb SET address=(?) WHERE username=(?)",
                    (address, username.lower()))
        if 'phoneNumber' in kwargs.keys() and kwargs['phoneNumber'] is \
                not None:
            c.execute("UPDATE staffdb SET phone=(?) WHERE username=(?)",
                    (phoneNumber, username.lower()))
        if 'email' in kwargs.keys() and kwargs['email'] is not None:
            c.execute("UPDATE staffdb SET email=(?) WHERE username=(?)",
                    (email, username.lower()))
        if 'priv' in kwargs.keys() and kwargs['priv'] is not None:
            c.execute("UPDATE staffdb SET priv=(?) WHERE username=(?)",
                    (priv, username.lower()))
        conn.commit()
        conn.close()
        return True

    def delete_user(self, username):
        """
        This method should only accessable by system admin, it's not frequently
        used anyway.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        c.execute("DELETE FROM userdb WHERE username=(?)",(username.lower(),))
        conn.commit()
        conn.close()

    def delete_staff(self, username):
        """
        This method should only be accessable to system admins.
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        c.execute("DELETE FROM staffdb WHERE username=(?)",(username.lower(),))
        conn.commit()
        conn.close()

    def authenticate(self, username, password, flag=0):
        """
        Authenticate a user's username and password.
        
        Arguments: username, password and a flag.

        Flag when 0 (default), customer login.
                  1, staff login

        Returns: True on successful authentication, 
                 False on user not exist or password incorrect.
        """
        enteredPassword = self.__sha512_hexdigest(password)
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        if flag is 0:
            dbpw = c.execute("SELECT password FROM userdb WHERE username=(?)",
                    (username.lower(),))
        elif flag is 1:
            dbpw = c.execute("SELECT password FROM staffdb WHERE \
                    username=(?)", (username.lower(),))
        else:
            conn.close()
            return False

        try:
            hashed = dbpw.fetchone()
        except StopIteration:
            conn.close()
            return False
        else:
            conn.close()
            try:
                ret = hashed[0]
            except TypeError:
                return False
            else:
                return hashed[0] == enteredPassword

    def get_staff_privilege(self, username):
        """
        This should be used after authentication of a staff is done. It will
        return the privilege value of a staff.

        Privilage is the same as add_staff:

        0 as system admin (full privilage)
        1 as normal staff (cashier)
        2 as stocking staff (add/remove DVD stock)
        """
        conn = sqlite3.connect(dbpath)
        c = conn.cursor()
        raw = c.execute("SELECT priv FROM staffdb WHERE username=(?)",
                (username.lower(),))
        try:
            priv = raw.fetchone()
        except StopIteration:
            conn.close()
            return None
        except TypeError:
            conn.close()
            return None
        else:
            conn.close()
            return priv[0]
